﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("SolutionDictionary", "Stores solutions generated by an algorithm.")]
  [StorableClass]
  [NonDiscoverableType]
  public abstract class SolutionDictionary<TKey, TValue> : Item {
    [Storable]
    protected Dictionary<TKey, List<TValue>> solutionDictionary;

    public SolutionDictionary() {
      solutionDictionary = new Dictionary<TKey, List<TValue>>();
    }

    [StorableConstructor]
    protected SolutionDictionary(bool deserializing) : base(deserializing) { }
    protected SolutionDictionary(SolutionDictionary<TKey, TValue> original, Cloner cloner)
      : base(original, cloner) {
      this.solutionDictionary = new Dictionary<TKey, List<TValue>>(original.solutionDictionary);
    }

    public void Add(TKey key, TValue value) {
      if (solutionDictionary.Count == 0) {
        var lst = new List<TValue>();
        lst.Add(value);
        solutionDictionary.Add(key, lst);
        return;
      }

      if (ContainsKey(key)) {
        if (!solutionDictionary[key].Contains(value))
          solutionDictionary[key].Add(value);
      } else {
        InsertKeyValue(key, value);
      }
    }

    protected abstract void InsertKeyValue(TKey key, TValue value);

    public virtual bool ContainsKey(TKey key) {
      return solutionDictionary.ContainsKey(key);
    }
  }
}
